function C=poleplace(varargin)
%C = poleplace(P,poles) is a proper controller
%which drives the poles of the closed-loop
%system (P,C) onto 'poles' which is a vector 
%consisting of all desired poles. 'P' should
%be a tf object and 'poles' should be a vector.
%
%C = poleplace(P,poles,'strict') is a strictly
%proper controller instead of a proper one.
%
%Programmed by WAN Shuang, HKUST, Mar 2009.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input format check & regulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin<2 || nargin>3
    error('Too many or too few input arguments');
end

if nargin==3
    if strcmp(varargin{3},'strict')
        isstr=1;
    else
        error('Invalid function option');
    end
else
    isstr=0;
end

if ~isa(varargin{1},'tf')
    error('P(s) should be a tf object');
else
    P=varargin{1};
end

if ~isscalar(P)
    error('P(s) should be scalar');
end

if ~isvector(varargin{2})
    error('Poles should be given in a vector');
else
    poles=varargin{2};
end

if ~isnumeric(poles)
    error('Poles could only be numeric');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Diophantine equation calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c=poly(poles); % Characteristic polynomial
n=length(P.den{1})-1;
m=length(c)-n-1;

if isstr % If strictly proper controller is desired
    if n>m
        error('n>m, too few poles to assure a strictly proper controller');
    elseif n<m
        warning('n<m, strictly proper controller is not unique');
    end
    controller(1)=c(1)/P.den{1}(1); % p_0=c_0/a_0
    S=[toeplitz([P.den{1} zeros(1,m-1)],[P.den{1}(1) zeros(1,m-1)]),toeplitz([P.num{1} zeros(1,m-1)],[P.num{1}(1) zeros(1,m-1)])]; % Sylvester matrix
    controller=[controller;S\(c(2:length(c)).'-controller*[P.den{1}(2:n+1).';zeros(m,1)])]; % Diophantine equation
else % If proper controller is desired
    if n>m+1
        error('n>m+1, too few poles to assure a proper controller');
    elseif n<m+1
        warning('n<m+1, proper controller is not unique');
    end
    S=[toeplitz([P.den{1} zeros(1,m)],[P.den{1}(1) zeros(1,m)]),toeplitz([P.num{1} zeros(1,m)],[P.num{1}(1) zeros(1,m)])];% Sylvester equation
    controller=S\(c.'); % Diophantine equation
end

controller=controller.'; % Convert to row vector s.t. compatible to tf()
C=tf(controller(m+2:length(controller)),controller(1:m+1));