function varargout=routh(varargin)
%stability = routh(p) is true if the given numeric 
%polynomial 'p' is stable, which is judged by its Routh
%table. Moreover the table is listed explicitly.
%
%[stability r] = routh(p) not only tells the stability
%of the polynomial 'p', but also stores the Routh table 
%in (the numeric) 'r'.
%
%first_column = routh(p_sym) is the first column of the
%Routh table, if one of the elements of 'p_sym' is
%symbolic. Corresponding Routh table is also drawn.
%
%[...] = routh(p,'silent') does the same job as the one
%without parameter 'silent' except that the Routh table
%is not listed.
%
%Programmed by WAN Shuang, HKUST, Mar 2009.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input format check & regulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==2
    if ischar(varargin{2})
        issilent=strcmp(varargin{2},'silent');
    else
        error('The 2nd input can only be "silent"');
    end
else
    issilent=0;
end

p=varargin{1};    

if isnumeric(p)||isa(p,'sym')
    if size(p,1)~=1
        if size(p,2)~=1
            error('Require vectorial inputs!');
        else
            p=p.';
            warning('Column vector input detected, convert to row vector automatically.');
        end
    end
else
    error('Require numerical or symbolic inputs!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Routh table calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rw=length(p); % Max row number of the Routh table
isodd=mod(rw,2); % Whether p is of odd number-th order
cl=(rw+isodd)/2; % Max column number of the Routh table
r=zeros(rw,cl); % An empty Routh table
if isodd
    p=[p 0]; % Make sure that p is compatible to reshape()
end
if isa(p,'sym')
    r=sym(r); % Make empty table compatible to symbols
end

r([1,2],:)=reshape(p,2,cl); % Assign the initial two rows

for k=3:rw % Calculate the descendent rows
    r(k,:)=[r(k-2,2:cl) 0]-r(k-2,1)/r(k-1,1)*[r(k-1,2:cl) 0];
end

if isnumeric(p)
    stability=prod((r(:,1)>0).^2); % stability
    if ~stability
        semi=~(sum(r(:,1)<0)); % semi-stability
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output variable assignment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isnumeric(p) % when p is numeric
    if nargout<=2
        varargout(1)={stability};
        if nargout==2
            varargout(2)={r};
        end
    elseif nargout>2
        error('1 or 2 outputs for numeric polynomials');
    end
else % then p is symbolic
    varargout(1)={simple(r(:,1))};
    if nargout>1
        error('only 1 output for symbolic polynomials');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Routh table drawing (negligable)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
if ~issilent % Then draw the Routh table
    if isnumeric(p) % Numeric case
        table=num2str(r);
    else % Symbolic case
        table=cell2mat(mat2cell([char(cell(r)) blanks(rw*cl)' blanks(rw*cl)'],rw*ones(1,cl))');
    end
   
    table=[strrep(blanks(rw),' ','s')' strrep(blanks(rw),' ','^')' num2str((rw-(1:rw)+1)') blanks(rw)' strrep(blanks(rw),' ','|')' blanks(rw)' table];  % Decorations
    table=[strrep(blanks(size(table,2)),' ','=');table]; % Table top cover    
    
    disp(' '); % Blank line
    disp(table);% Draw decorated Routh table

    if isnumeric(p) % The conclusion on stability, only for numeric case
        table=''; % Variable reuse
        for k=1:rw % Explicit polynomial description
            if p(k)~=0
                if k~=1 && p(k)>0
                    table=[table,' + '];
                end
                if p(k)~=1 || k==rw
                    table=[table,num2str(p(k))];
                end
                if k==rw-1
                    table=[table,'s'];
                elseif k<rw-1
                    table=[table,'s^',num2str(rw-k+1)];
                end            
            end
        end
        conclusion=sprintf('The polynomial %s is ',table);

        if stability
            table='stable.';
        else
            table='not stable';
            if semi
                table=[table ' (but semi-stable)'];
            end
            table=[table '.'];
        end

        conclusion=[conclusion table]; % Final conclusion on stability
        disp(' '); % Blank line
        disp(conclusion);
        disp(' '); % Blank line
    end % Conclusion display finishes
end % Non-silent display finishes